package spamfilter;

import java.io.*;
import java.util.*;

/**
  * A command/data processor for a conversation between a client and a
  * mailserver. Each time <code>process</code> is called, the internal
  * <code>state</code> of the <code>Analyzer</code> may be changed, allowing
  * certain commands to be ignored/passed on while others are parsed, possibly
  * capturing interesting data for further use. The
  * <code>blockClientRequests</code> flag serves as an indicator to external
  * code that a client command should NOT be passed to the <code>Analyzer</code>
  * in its current <code>state</code>, as the client-mailserver conversation may
  * become corrupted.
  * @author Jennifer Knutson
  */
public abstract class Analyzer { 

    /**
      * The internal state of this object affecting the processing of commands.
      * The default, initial state for all subclasses should be <b>0</b>.
      */
    protected int state;

    protected boolean blockClientReq;
    protected StringBuffer mailBuf;
    protected Hashtable sessionMail = null;
    
    public abstract void process(String cmd, ChannelInfo chInfo)
	throws IOException; 
    
    public Analyzer() {
	state = 0;
	mailBuf = new StringBuffer(64000);
	sessionMail = new Hashtable(400);
    }

    public boolean blockClientRequests() {
	return blockClientReq;
    }

    protected void reset() {
	state = 0;
	blockClientReq = false;
	if(mailBuf != null)
		mailBuf = mailBuf.delete(0, mailBuf.length());	
    }


    // GUI display
    public void showFilteredMessages() {	
	Vector v = new Vector(sessionMail.values());
	TrainingWindow tw;
	if(v.size() > 0)
	    tw = new TrainingWindow(v);
    }

    // text display
    /*
    private void listMessages(Hashtable t, String s) {	
	System.out.println(s);
	Vector v = new Vector(t.values());
	for(int i = 0; i < v.size(); i++) {
	    Email msg = (Email) v.get(i);    
	    System.out.println("---> " + msg.getSender()			
		    + "\n" + msg.isSpam() + " " + msg.getSubject());	
	}	
    }
    */
}
