/*
     CorpusTokenCount.java by Matt Hansen and Tom Ashley for CS400
*/
package spamfilter;
import java.io.*;
import java.util.*;

/**
 * Tokenizes a text file and produces a hashtable mapping tokens
 * to the frequency of its occurence.  Tokens located in the header fields
 * of an email are given a special classification: headername*token.
 * Tokens consist of alphanumeric characters, $, ', -, :, as suggested
 * by Paul Graham.
 * @author Matt Hansen
 * @author Tom Ashley
 */
public class CorpusTokenCount
{
     /**
      * Here's the main method. 
      * @param args input file name and output file name
      */
     public static void main(String[] args) throws IOException
     {
          int numMessages = 0;
          if (args.length != 2)
          {
               System.err.println("Wrong number of arguments");
               System.err.print("Format is: input file name, ");
               System.err.println("output file name");
               System.exit(1);
          }

          Hashtable tokenlist = new Hashtable(6000);

          try
          {
               FileReader fr = new FileReader(args[0]);
               BufferedReader br = new BufferedReader(fr);
               StreamTokenizer st = new StreamTokenizer(br);
               st.eolIsSignificant(true);
               st.resetSyntax();

               // Characters to be considered as parts of tokens
               st.wordChars('A', 'Z');
               st.wordChars('a', 'z');
               st.wordChars('0', '9');
               st.wordChars('!', '!');
               st.wordChars('$', '$');
               st.wordChars('-', '-');
               st.wordChars(':', ':');
               //st.wordChars(10, 10); // line-feed
               //st.wordChars(13, 13); // carriage-return
               st.wordChars(39, 39); //Include apostrophe in token
               int type;

               boolean delim = false;
               String header = "";
               while ((type = st.nextToken()) != StreamTokenizer.TT_EOF)
               {
                   if (type == StreamTokenizer.TT_EOL)
                        header = "";
                   if (type == StreamTokenizer.TT_WORD)
                   {
                      String token = st.sval.toString();
                      if (token.equals("To:")) header = "to*";
                      else if (token.equals("From:"))
                      {
                         numMessages++;
                         header = "from*";
                      }
                      else if (token.equals("Return-Path:"))
                         header = "return_path*";
                      else if (token.equals("Subject:"))
                         header = "subject*";
		      else if(tokenlist.containsKey(header+st.sval))
                      //increment the count
		      {
                         Integer temp = 
                            (Integer)tokenlist.get(header+st.sval);
                         int value = temp.intValue();
                         value++;
                         tokenlist.put(header+st.sval, new Integer(value));
	              }
		      else
                      //set the count to 1
		      {
                         tokenlist.put(header+st.sval, new Integer(1));
		      }
		   }
               }
               br.close();
          }
          catch (IOException e)
          {
               System.err.println(e);
          }

          FileOutputStream out = new FileOutputStream(args[1]);
          ObjectOutputStream s = new ObjectOutputStream(out);
          s.writeObject(new Integer(numMessages));
          s.writeObject(tokenlist);
          s.close();
          out.close();
     }
}
