// SPAM_TAG & NONSPAM_TAG should really be in globals!!!!


package spamfilter;

import java.io.*;
import java.util.*;


public class Email implements Serializable {

    // Email message headers (name & data)
    public class MailHeaderField implements Serializable{
	private String name; // header field name
	private String data; // header contents
    
	public MailHeaderField(String fieldName, String fieldData) {
	    name = new String(fieldName);
	    data = new String(fieldData);
	}

	public void append(String s) {
	    data += (Globals.CRLF + s);
	}

	public String toString() {
	    return (name + ": " + data);
	}
	
	public String headerType() {
	    return name;
	}

	public String headerData() {
	    return data;
	}

	// Add string t to front of data
	public String toStringTagged(String t) {
	    return (name + ": " + t + " " + data);
	}
    }

    // ***** Email class ***** 
    private int index = -1; // email header index
    private Vector headers; // collection of email headers
    private StringBuffer body; // message body
    private int messageID; // message number
    private int	subjectIndex = -1;
    private int	fromIndex = -1;
    private boolean spam = false;
    private static final String SPAM_TAG = "[SPAM]";
    private static final String NONSPAM_TAG = "[NONSPAM]";
    
    // constructor
    public Email(String rawMail, int msgID) 
	throws IOException {
	
	// Initialize variables
	headers = new Vector();
	body = new StringBuffer(10000);
	messageID = msgID;
	index = 0;

	BufferedReader in = new BufferedReader(new StringReader(rawMail));
	String s = null; // a line of rawMail
	boolean msgBody = false;
	MailHeaderField currentHeader = null;
	
	int headerCount = 0;
	// Read in rawMail line-by-line, putting data in appropriate fields
	while((s = in.readLine()) != null) {
		
	    if(!msgBody && s.length() == 0) {
		// blank line = start of body
		msgBody = true;
	    }
	    else if (!msgBody) { // still fetching headers	       
		String[] a = s.split(": ", 2); // Break off stuff before colon
		   

		if(a != null && a.length > 1 
			&& !Character.isWhitespace(s.charAt(0))){ 
	
		    currentHeader = new MailHeaderField(a[0], a[1]);
		    headers.add(currentHeader);

		    if(a[0].equalsIgnoreCase("Subject"))
			subjectIndex = headerCount;
		    else if(a[0].equalsIgnoreCase("From"))
			fromIndex = headerCount;

		    headerCount++;
		} else if(currentHeader != null && a != null)  {
		    currentHeader.append(a[0]);	
		}

// *****DEBUGGING***
		else
		    System.err.println("ERROR in class Email; ignored: " + s);
// *********

	    } else { // message body; add stripped CRLF back to end of line
		body.append(s + Globals.CRLF);
	    } 
	}
    }

    
    // return Email object as StringBuffer; bodyLines specifies how many lines
    // of the body are returned (a negative number indicates all lines should be
    // returned as part of the String)
    public StringBuffer asPopMail(int bodyLines, boolean showTag) throws
	IOException { 
	StringBuffer rawMail = new StringBuffer();

	// Add headers to StringBuffer
	for(int i = 0; i < headers.size(); i++) {
	 
	    if(showTag && i == subjectIndex)
		rawMail.append(
			((MailHeaderField)headers.get(i)).toStringTagged(tag())
			+ Globals.CRLF);
	    else
		rawMail.append(((MailHeaderField)headers.get(i)).toString() +
		    Globals.CRLF);
	}
	
	// blank line before body
	rawMail.append(Globals.CRLF);

	if(bodyLines < 0) // append entire message body
	    rawMail.append(body);
	else if(bodyLines > 0) { // append only some lines of body (TOP)
	    BufferedReader in = new BufferedReader(new
		    StringReader(body.toString()));
	    String s = null; // a line from body of message
	    int i = 0;

	    while((s = in.readLine()) != null && i < bodyLines) {
		rawMail.append((s + Globals.CRLF));
		i++;
	    }
	}
	// else: bodyLines == 0; don't append message body at all
	
	return rawMail;
    }


    private String tag() {
	if(spam)
	    return SPAM_TAG;
	else
	    return NONSPAM_TAG;
    }
    
    // reset headers Vector index to zero
    public void resetHeaderIndex() {
	index = 0;
    }
    
    // true while there are mail header fields left to read
    public boolean hasNextHeader() {
	if((index + 1) < headers.size())
	    return true;
	else
	    return false;
    }
    
    // retrieve next mail header field
    public MailHeaderField getNextHeader() throws IOException {
	if(hasNextHeader())
	    return (MailHeaderField)headers.get(index++);
	else
	    throw new IOException("Ran out of email headers to retrieve.");
    }

    // retrieve email message body
    public String getMessageBody() {
	return body.toString();
    }

    public String getSubject() {
	if(subjectIndex < 0)
	    return "";
	else
	    return ((MailHeaderField)headers.get(subjectIndex)).headerData();
    }

    public String getSender() {
	if(fromIndex < 0)
	    return "";
	else
	    return ((MailHeaderField)headers.get(fromIndex)).headerData();
    }

    public int getID() {
	return messageID;
    }

    public void markAsSpam() {
	spam = true;
    }

    public void markAsNonspam() {
	spam = false;
    }

    public boolean isSpam() {
	return spam;
    }
}
