/*
   GetProb.java by Tom Ashley and Matt Hansen for CS 400 -- SpamFilter
*/

package spamfilter;
import java.io.*;
import java.util.*;

/**
 * Calculates the probability that a token is spam.  This class takes
 * a hashtable of spam token counts and a hashtable of nonspam token
 * counts, and combines these values to create a spam probability for each
 * token.  The token and its associated probability are mapped to each
 * other in a hashtable.  Algorithm from Paul Graham. The hashtables are
 * obtained from the Globals file: Globals.spamCorpus and
 * Globals.nonspamCorpus.
 * @author Tom Ashley
 * @author Matt Hansen
 */
class GetProb
{
    public static void main(String[]args)
    {
	Hashtable spam = null;
	Hashtable nonspam = null;
	int nbad = 0;
	int ngood = 0;

        //Read in the spam corpus and the # of spam messages
        try {
	   FileInputStream in = new FileInputStream(Globals.spamCorpus);
	   ObjectInputStream s = new ObjectInputStream(in);
	   nbad = ((Integer) s.readObject()).intValue();
	   spam = (Hashtable) s.readObject();
	   s.close();
	   in.close();
	} catch (FileNotFoundException e) {
           System.out.print("Spam corpus file not found:");
           System.out.println(" Creating empty file");
           try {
              ObjectOutputStream out = new ObjectOutputStream(
                 new FileOutputStream(Globals.spamCorpus));
              out.writeObject(new Hashtable());
              out.close();
           } catch (IOException ex) {
              System.err.println(ex);
           }
        } catch (IOException e) {
           System.err.println(e);
        } catch (ClassNotFoundException r) {
           System.err.println(r);
        }

        //Read in the nonspam corpus and the # of nonspam messages
        try {
	   FileInputStream in2 = new FileInputStream(Globals.nonspamCorpus);
	   ObjectInputStream s2 = new ObjectInputStream(in2);
	   ngood = ((Integer) s2.readObject()).intValue();
	   nonspam = (Hashtable) s2.readObject();
	   s2.close();
	   in2.close();
	   
	   if(spam != null && nonspam != null)
	       updateProbCount(spam, nonspam, nbad, ngood);
	}
        catch (FileNotFoundException e) {
           System.out.print("Non spam corpus file not found:");
           System.out.println(" Creating empty file");
           try {
              ObjectOutputStream out2 = new ObjectOutputStream(
                 new FileOutputStream(Globals.nonspamCorpus));
              out2.writeObject(new Hashtable());
              out2.close();
           } catch (IOException ex) {
              System.err.println(ex);
           }
        }
        catch (IOException e)
        {
           System.err.println(e);
        }
	catch (ClassNotFoundException r)
	{
	   System.err.println(r);
	}
    }
    
     /**
      * Go through each token in the spam hashtable.  Find its
      * frequency in spam and nonspam, and calculate the probability
      * based on these values plus the total number of messages in the
      * spam and nonspam corpora.  Cross the token off the nonspam list.
      * After looking at all the spam tokens, go through the remaining
      * nonspam tokens and assign them probabilities of 0.  (Assuming
      * that they've occured at least three time.)  In general, a token
      * must occur at least five times in the spam and nonspam corpora
      * to be given a probability.
      * @param spam hashtable containing counts of tokens in spam messages
      * @param nonspam hashtable containing counts of tokens in nonspam
      * messages
      * @param nbad number of messages in the nonspam corpus
      * @param ngood number of messages in the spam corpus
      */
     public static void updateProbCount(Hashtable spam, Hashtable nonspam, 
					int nbad, int ngood) 
	 throws IOException
     {
          //Get occurrences of token in spam = b
          //g = 2*occurrences in non-spam corpus
          //Only calculate probabilities for tokens where g+b > 5
          //Probabilities must be between .01 and .99
          //Probability = (b / nbad) / ((g / ngood) + (b / nbad))

          try
          {
               String token;
               int count, bad, good; 
               float prob;
               Integer temp;

               Enumeration spamkeys = spam.keys();
               Enumeration spamcounts = spam.elements();
               Hashtable problist = new Hashtable(10000);
               while (spamkeys.hasMoreElements())
               {
                    //Get the number of occurrences in the non-spam table
                    token = (String) spamkeys.nextElement();
                    temp = (Integer) spamcounts.nextElement();
                    bad = temp.intValue();
                    if ((temp = (Integer) nonspam.get(token)) != null)
                         good = 2*temp.intValue();
                    else
                         good = 0;

                    //Calculate probabilities if good + bad > 5
                    if (good + bad > 5)
                    {
                          float numerator = (float) bad / (float) nbad;
                          if (numerator > 1.0) numerator = (float) 1.0;
                          float denom = (float) good / (float) ngood;
                          if (denom > 1.0) denom = (float) 1.0;
                          prob = numerator / (numerator + denom);

                          // Token probability must be between .01 and .99
                          if (prob < 0.01) prob = (float) 0.01;
                          if (prob > 0.99) prob = (float) 0.99;
                          Float probability = new Float(prob);
                    
			  problist.put(token, probability);
                    }

                    if (good != 0) nonspam.remove(token);
                    // We've already looked at this token, so we won't
                    // need to evaluate it when we enumerate the
                    // nonspam corpus
               }

               Enumeration nonspamkeys = nonspam.keys();
               Enumeration nonspamcounts = nonspam.elements();
               while (nonspamkeys.hasMoreElements())
               {
                    //Set spam probability to 0.01, if 2*good > 5
                    token = (String) nonspamkeys.nextElement();
                    temp = (Integer) nonspamcounts.nextElement();
                    good = 2*temp.intValue();
                    Float zero = new Float(0.01);
                    if (good > 5)
                         problist.put(token, zero);
               }

               FileOutputStream out = new FileOutputStream(
		       Globals.probTableFile);
               ObjectOutputStream s3 = new ObjectOutputStream(out);
               s3.writeObject(problist);
               s3.close();
               out.close();
          }
          catch (IOException e)
          {
               System.err.println(e);
          }
	  //          catch (ClassNotFoundException r)
          //{
	  //   System.err.println(r);
          //}
     }
}
