package spamfilter;

import java.io.*;
import java.util.*;
import java.util.regex.*;

public class IMAPAnalyzer extends Analyzer { 

    // From class Analyzer:
    // * int state
    // * boolean blockClientReq
    // * StringBuffer mailBuf  

    //recentStatusPattern and recentStatusMatcher are used
    //to get the number of Recent messages from the server (into numRecent)
    Pattern noopRecentPattern = 
	Pattern.compile("(?i)\\* (\\d+) RECENT.*[\\n\\r]*");
     Pattern recentStatusPattern =
    Pattern.compile("(?i)\\* STATUS \"inbox\" \\(RECENT (\\d+)\\)[\\n\\r]*");
    Pattern recentPattern = 
	Pattern.compile("(?i)\\* ([\\d]*) RECENT[ \\n\\r]*");
    Pattern uidListPattern = 
	Pattern.compile("(?i)\\* SEARCH ([ \\d]*)[\\n\\r]*");
    Pattern selectPattern = 
	Pattern.compile("(?i)\\w* SELECT ([\\w\\p{Punct}]*)[ \\n\\r]*");
    Pattern selectResponse =
	Pattern.compile("(?i)\\w* SELECT \\[READ-ONLY\\][ \\w]*[ \\n\\r]*");
    Pattern selectStatusPattern =
	Pattern.compile("(?i)\\* (\\d+) RECENT.*[\\n\\r]*");
    Matcher selectStatusMatcher;
    Matcher selectResponseMatcher;
    Matcher selectMatcher;
    Matcher noopRecentMatcher;
    Matcher recentStatusMatcher;
    Matcher recentMatcher;
    Matcher uidListMatcher;//do we need four matchers or can we use 1?
    int numRecent;
    String[] recentMessages; //holds the strings of uids for recent messages
    int commandNumber;
    String clientCommand;
    int messagePointer;
    int waitstate;
    String flagToSet;
    String messageNumbers;//all recent message uids in one string
    String currentMailbox; //Mailbox that is selected
    String noResponse = "(?i)SB[\\w\\d]+ NO.*[\\n\\r]*";
    String badResponse = "(?i)SB[\\w\\d]+ BAD.*[\\n\\r]*";
    boolean selected = false;
    String serverResponse = new String();
    int lastChecked = 0;
    String oKResponse = "(?i)[\\w\\d]+\\s+OK.*[\\n\\r]*";
    
    /**
     *
     *Monitors client/server communications to determine when 
     *the inbox, and then blocks communication and runs any new messages
     *through <code>SpamIdentifier</code>.  If messages are identified as
     *spam, they are deleted or marked on the server.
     *Any messages that arrive during the session are also checked.
     *@author Marie Joiner and Josh Ourisman
     */

    public IMAPAnalyzer() {
	super();
	state = 0;//keeps track of what the analyzer should do next
	blockClientReq = false;/*allows the proxy to talk with the
	//server without the silly client interfering, for example 
	//when we are checking recent messages for spam  value*/
	commandNumber = 0;//keeps track of line numbers of server
	//requests
	numRecent = 0;
	waitstate = 7;
	messagePointer = 0;
	if(Globals.spamAction == 1) flagToSet = "(\\Flagged)";
	else if (Globals.spamAction == 2) flagToSet = "(\\Deleted)";
    }
    
  //process looks at the current command, coming from either 
    //the client or the server, and decides if it is important
    // in the marking of spam
    /**
     *Parses current command,<code>cmd</code> coming from either the client o
     *the server, and decides if it is relevant in marking spam.
     *Server responses to <code>IMAPAnalyzer</code> responses
     *are not passed to client.  All other <code>cmd</code> (commands)
     *are passed through after being parsed.
     *@param chInfo The channel that is being "monitored."
     *@param cmd String sent from either client or server.
     */
    public void process(String cmd, ChannelInfo chInfo) throws IOException {

	
	// #### debug ########################
	if(chInfo.isClient())
	    System.out.print("C[");
	else
	    System.out.print("S[");
	System.out.print(state + "]: " + cmd);
	// ####################################

	

	
	switch(state){
	case 0:
	    //PREVIOUSLY: nothing
	    //WAITING FOR: a select mailbox from client
	    //DOES: goes to state 1

	    selectMatcher = selectPattern.matcher(cmd);    
	    if (selectMatcher.matches()) {
		currentMailbox = selectMatcher.group(1);
		blockClientReq = true;
		state = 1;
		chInfo.write(cmd);
	    }
	    else
		chInfo.write(cmd);
	    break;
	    
	case 1:
	    //PREVIOUSLY: selected mailbox
	    //WAITING FOR: response and OK from server
	    //DOES:matches number Recent, sends a No-op, goes to state 2
	    //ALTERNATE: state 0 for read-only mailbox

	    selectStatusMatcher = selectStatusPattern.matcher(cmd);
	    if (selectStatusMatcher.matches()) {
		serverResponse = serverResponse + cmd;
		numRecent =
		    Integer.decode(selectStatusMatcher.group(1)).intValue();
		// System.out.println("Number of recent: " + numRecent);

	    }
	    else if(cmd.matches(oKResponse)){
		serverResponse = serverResponse + cmd;
		selectResponseMatcher = selectResponse.matcher(cmd); 
		if(selectResponseMatcher.matches())
		    {
			//System.out.println("Mailbox read only.");
			state = 0;
		    }
		else {
		    state = 2;
		    System.out.println("P[" + state + "]: SB"
				       + commandNumber + " NOOP");
		    chInfo.writeToServer("SB" + commandNumber++ + " NOOP" +
					 Globals.CRLF);
		}
	
	    }
	    else if(cmd.matches(noResponse)||cmd.matches(badResponse)){
		//When an invalid mailbox is selected, NO mailbox
		//is selected, it doesn't go back to the previous one
		//shouldn't ever happen -- error message?
		System.out.print("P[" + state + "] (clientCommand): " 
				 + clientCommand);
		chInfo.writeToServer(clientCommand);
		resetAnalyzer();
	    }
	    else
		serverResponse = serverResponse + cmd;
	    break;

	case 2:
	    //PREVIOUSLY: no-op
	    //WAITING FOR: # recent and OK from server
	    //DOES: records # recent and, if greater than 0, 
	    //requests uids, otherwise,just goes to wait state
	    
	    noopRecentMatcher = noopRecentPattern.matcher(cmd);
	    if(noopRecentMatcher.matches()){
		numRecent = 
		    Integer.decode(noopRecentMatcher.group(1)).intValue();
		//System.out.println("Number of recent: " + numRecent);
		
	    }
	    else if(cmd.matches("(?i)SB\\d+ OK STATUS.*[\\n\\r]*") ||
		    cmd.matches("(?i)SB\\d+ OK NOOP completed.*[\\n\\r]*")) {

		if(numRecent == 0){
		    //no messages to be analyzed, allow client
		    //to continue
		    state = waitstate;
		    blockClientReq = false;
		    if (!selected) {
			System.out.print("P[" + state + "] (SR): " + serverResponse);
			chInfo.writeToClient(serverResponse);
			selected = true;
		    }
		    else {
			System.out.print("P[" + state + "] (CC): " 
					 + clientCommand);
			chInfo.writeToServer(clientCommand);
		    }

		}
		else if(cmd.matches(noResponse)||cmd.matches(badResponse)){
		    System.out.print("P[" + state + "] (CC): " 
				     + clientCommand);
		    chInfo.writeToServer(clientCommand);
		    resetAnalyzer();
		}
		else{
		    state = 3;
		    //there are messages to be analyzed!
		    System.out.println("P[" + state + "]: SB" + 
				       commandNumber + " uid search recent");
		    chInfo.writeToServer("SB" + commandNumber++ + 
					 " uid search recent" + Globals.CRLF);
		}
	    }
	    break;
	    
	case 3:
	    //PREVIOUSLY: searched for uid recents
	    //WAITING FOR: numbers of uids and OK from server
	    //DOES: stores uids and requests 1st header after the last
	    //message we've already analyzed

	    uidListMatcher = uidListPattern.matcher(cmd);
	 
	    //puts message numbers into an array
	    if(uidListMatcher.matches()){
		messageNumbers = uidListMatcher.group(1);
		//		System.out.println("Here are message numbers: "
		//	   + messageNumbers);
		recentMessages = messageNumbers.split("\\s+");
		messagePointer = 0;

		for(int j = 0;  j < (recentMessages.length); j++){
		    if (Integer.parseInt(recentMessages[j]) <= lastChecked)
			messagePointer++;
		}
		//System.out.println("Messsage pointer:"+ messagePointer);
	    }
	    //requests a message header
	    else if(cmd.matches("(?i)SB\\d+ OK UID SEARCH.*[\\n\\r]*") ||
		    cmd.matches("(?i)SB\\d+ OK SEARCH.*[\\n\\r]*")) {
		if(messagePointer >= recentMessages.length){
	  
		    state = waitstate;
		    System.out.print("P[" + state + "] (CC): " 
				     + clientCommand);
		    chInfo.writeToServer(clientCommand);
		    blockClientReq = false;
		}
		else
		    {
			state = 4;
			if(mailBuf != null) // clear mailBuf for new message
			    mailBuf.delete(0, mailBuf.length());
			
			System.out.println("P[" + state + "]: SB" 
					   + commandNumber + " uid fetch " +
					   recentMessages[messagePointer] +
					   " body.peek[header]");	
			chInfo.writeToServer("SB" + commandNumber++ +
					     " uid fetch " +
					     recentMessages[messagePointer] +
					     " body.peek[header]" +
					     Globals.CRLF);
			//DO NOT INCREASE messagePointer, because we need
			//the message body of the SAME message	
		    }
	    }
	    else if(cmd.matches(noResponse) || cmd.matches(badResponse)){
		    System.out.print("P[" + state + "] (CC): "
				     + clientCommand);
		chInfo.writeToServer(clientCommand);
		resetAnalyzer();
	    }
	    break;
	    
	case 4:
	    //PREVIOUSLY: requested message header from server
	    //WAITING FOR: header and OK from server
	    //DOES:stores headers and requests body

	    if(cmd.matches("(?i)SB\\d+ OK UID FETCH.*[\\n\\r]*") ||
	       cmd.matches("(?i)SB\\d+ OK FETCH.*[\\n\\r]*")) {
	System.out.print(cmd);

		state = 5;
		//Since we are fetching the body, we use the same
		//messagePointer as for the header
		System.out.println("P[" + state + "]: SB" + commandNumber + 
				   " uid fetch " +
				   recentMessages[messagePointer] + 
				   " body.peek[text]");
		chInfo.writeToServer("SB" + commandNumber++ + " uid fetch " +
				     recentMessages[messagePointer] + 
				     " body.peek[text]" + Globals.CRLF);
	    }
	    else if(cmd.matches("(?i)\\* \\d+ FETCH \\(UID \\d+ BODY\\[\\w+\\].*[\\n\\r]*") ||
		    cmd.equals(")" + Globals.CRLF) ||
		    cmd.matches("(?i)\\* \\d+ FETCH \\(FLAGS.*\\)[\\n\\r]*")||
		    cmd.matches("(?i)SB\\d+ STORE [\\w \\.]*[\\n\\r]*")){
	    }
	    else if(cmd.matches(badResponse)|| cmd.matches(noResponse)){
		System.out.println("No/Bad response in state 4");
		System.out.print("P[" + state + "] (CC): " + clientCommand);
		chInfo.writeToServer(clientCommand);
		resetAnalyzer();
	    }
	    else
		mailBuf.append(cmd);
	    break;

	case 5:
	    //PREVIOUSLY: requested body from server
	    //WAITING FOR: body and OK from server
	    //DOES:sends whole message to be parsed, and
	    //requests header of next message, if there is one
	    //otherwise, goes to waitstate

	    //ignoring these lines, not part of message
	    if(cmd.matches("(?i)\\* \\d+ FETCH \\(UID \\d+ BODY\\[\\w+\\].*[\\n\\r]*") ||
	       cmd.matches("(?i)\\* \\d+ FETCH \\(FLAGS.*\\)[\\n\\r]*")||
	       cmd.matches("(?i)SB\\d+ STORE [\\w \\.]*[\\n\\r]*")){
	    }
	    
	    else if( cmd.equals(")"+ Globals.CRLF)){	
		Email currentEmail = parseMessage(mailBuf.toString(), recentMessages[messagePointer]);
		if (SpamIdentifier.isSpam(currentEmail)) {
		    currentEmail.markAsSpam();
		    System.out.println("Message " +
				       recentMessages[messagePointer] +
				       " is spam!");
		    //what flagToSet is DEPENDS on Globals.spamAction
		    System.out.println("P[" + state + "]: SB" + commandNumber +
		    	       " UID STORE " +
		    	       recentMessages[messagePointer] +
		    	       " +FLAGS.SILENT " + flagToSet);

		    chInfo.writeToServer("SB" + commandNumber++ +
					 " UID STORE " +
					 recentMessages[messagePointer] +
					 " +FLAGS.SILENT " + flagToSet +
					 Globals.CRLF);
		    }
		
		else {
		    currentEmail.markAsNonspam();
		    System.out.println("Message " +
				       recentMessages[messagePointer] +
				       " is not spam!");
		}

		sessionMail.put(new Integer(messagePointer), currentEmail);

		//It is DONE with this message, and ready for the next one
		messagePointer++;
		if(mailBuf != null) // reset message buffer
		    mailBuf.delete(0, mailBuf.length());

	    }

	    else if(cmd.matches("(?i)SB\\d+ OK UID FETCH.*[\\n\\r]*") ||
		    cmd.matches("(?i)SB\\d+ OK FETCH.*[\\n\\r]*")) {

		if(messagePointer < recentMessages.length)
		    {
			state = 4;
			System.out.println("P[" + state + "]: SB" 
					   + commandNumber + " uid fetch " +
				   recentMessages[messagePointer] + 
				   " body.peek[header]");
					    
			chInfo.writeToServer("SB" + commandNumber++ +
					     " uid fetch " +
					     recentMessages[messagePointer] + 
					     " body.peek[header]" + 
					     Globals.CRLF);
		    }
		else{
		    //no more messages to be analyzed, must select mailbox to
		    //clear number of recent
		    //keeps track of the last uid it analyzed.  any messages
		    //that arrive later MUST have a larger uid
		    lastChecked =
			Integer.parseInt(recentMessages[messagePointer-1]);
		    state = 6;
		    System.out.println("P[" + state + "]: SB" + commandNumber 
				       + " NOOP");
		    chInfo.writeToServer("SB" + commandNumber++ + " NOOP" 
		    		 + Globals.CRLF);
		}
	    }
	    else if(cmd.matches(noResponse) || cmd.matches(badResponse)){   
		System.out.print("P[" + state + "] (CC): " + clientCommand);
		chInfo.writeToServer(clientCommand);
		resetAnalyzer();
	    }
	    else
		mailBuf.append(cmd);
	    break;
	    
	case 6:
	    //PREVIOUSLY: sent a select mailbox
	    //WAITING FOR: OK from server
	    //DOES: opens it back up to client

	    if(cmd.matches("(?i)SB\\d+ STORE [\\w \\.]*[\\n\\r]*"));
	    else if(cmd.matches("(?i)SB[\\w\\d]+\\s+OK.*[\\n\\r]*")){
		state = waitstate;
		if (selected) {
		    System.out.print("P[" + state + "](CC): " + clientCommand);
		    chInfo.writeToServer(clientCommand);
		}
		else {
		    System.out.print("P[" + state + "](SR): "
				     + serverResponse);
		    chInfo.writeToClient(serverResponse);
		    selected = true;
		}
		blockClientReq = false;
	    }
	    
	    else if(cmd.matches(badResponse)||cmd.matches(noResponse)){
		System.out.print("P[" + state + "] (CC): " + clientCommand);
		chInfo.writeToServer(clientCommand);
		resetAnalyzer();
	    }
	    break;

	case 7:
	    //PREVIOUSLY: from many places, but no more messages were recent
	    //WAITING FOR: any command from client
	    //DOES: asks server if there are more recent messages using no-op

	    if (chInfo.isClient()) {
		if (cmd.matches("(?i)\\w+\\s+LOGOUT.*[\\n\\r]*"))
		    chInfo.write(cmd);
		else {
		    state = 2;
		    clientCommand = cmd;
		    blockClientReq = true;
		    
		    System.out.println("P[" + state + "]: SB" + 
				       commandNumber + " NOOP");
		    chInfo.writeToServer("SB" + commandNumber++ + " NOOP" +
					 Globals.CRLF);
		}
	    }
	    else if(!blockClientReq)
		    chInfo.write(cmd);
	
	    break;
	}
    }

    //makes a string (containing headers AND body)into an Email object    
    /**
     *Calls <code>Email</code> constructor on parameters.
     *@param uid Unique string id identifying message
     *@param message String holding entire message AND body
     *@return Returns an Email object.
     */
    private Email parseMessage(String message, String uid){
	try {	  
	    Email newmail = new Email(message, Integer.parseInt(uid));
	    return newmail;
	} catch (IOException e) {
	    e.printStackTrace();
	}
	return null;
    }

    /**
     *In case of bad/unexpected commands, critical variables are
     *reinitialized and client is unblocked.
     */
    private void resetAnalyzer(){
	state = 0;
	System.out.println("Resetting IMAP Analyzer!!!!!!!!!!!!!");
	System.out.println(clientCommand);
	blockClientReq = false;
    
	//probably unneccessary, but safe
	if(mailBuf != null)
	    mailBuf.delete(0, mailBuf.length());
	numRecent = 0;
	recentMessages = new String[0]; 
	messagePointer = 0;
	lastChecked = 0;
	messageNumbers = new String();
	currentMailbox = new String();
    }
    
}

