package spamfilter;

import java.io.*;
import java.util.*;

/**
 * Splits a corpus into ten components, for cross-validation.
 * Must be given the name of the file to be split, and whether the file
 * is spam or nonspam.  Each message is assigned a random float between
 * 0 and 1.  Depending on this value, a message is placed in one of ten
 * hashtables.  After all messages have been placed, the hashtables are
 * written to files.
 * @author Matt Hansen
 * @author Tom Ashley
 */
public class MakeTestFiles {
   public static void main(String[] args) throws IOException {
      
      int numMessages = 0;
      if (args.length != 2 || (!args[1].equals("spam") &&
                               !args[1].equals("nonspam")))
      {
         System.err.println("Wrong number of arguments");
         System.err.println("Format is: filename spam/nonspam");
         System.exit(1);
      }
      boolean isSpam = false;
      if (args[1].equals("spam"))
         isSpam = true;

      try {
         FileReader fr = new FileReader(args[0]);
         BufferedReader br = new BufferedReader(fr);
         String temp;
         StringBuffer msg = new StringBuffer(41024);
         Hashtable emailList = new Hashtable();
	 Random randSeq = new Random();
	 while ((temp = br.readLine()) != null)
	 {
	    if(temp.indexOf("From ") == 0)
	    {
	       if(numMessages != 0)
	       {
                  Email emsg = new Email(msg.toString(), numMessages);
	          if (isSpam)
                     emsg.markAsSpam();
                  else	  				  
		     emsg.markAsNonspam();
		  double randNum = randSeq.nextDouble();
	          Double d = new Double(randNum);
		  emailList.put(d, emsg);
		  msg.delete(0,(msg.length()-1));
	       }
	       numMessages++;
	    }
	    msg.append(temp + Globals.CRLF);
	 }
	 //We now have one big hashtable of Emails and rand numbers
		 
	 Enumeration randNumList = emailList.keys();
	 Enumeration msgList = emailList.elements();
	 Hashtable group1 = new Hashtable();
	 Hashtable group2 = new Hashtable();
	 Hashtable group3 = new Hashtable();
	 Hashtable group4 = new Hashtable();
	 Hashtable group5 = new Hashtable();
	 Hashtable group6 = new Hashtable();
	 Hashtable group7 = new Hashtable();
         Hashtable group8 = new Hashtable();
         Hashtable group9 = new Hashtable();
	 Hashtable group10 = new Hashtable();
	 Double dub = new Double(0.0);
	 Email temp2;
	 double d2;
	 while(randNumList.hasMoreElements())
	 {
            temp2 = (Email) msgList.nextElement();
            if (isSpam)
               temp2.markAsSpam();
            else
	       temp2.markAsNonspam();
	    //System.out.println(temp2.getMessageBody());
	    dub = (Double) randNumList.nextElement();
	    d2 = dub.doubleValue();
	    if((d2 >= 0.0) && (d2 <= 0.1))
	       group1.put(dub, temp2);
	    else if((d2 > 0.1) && (d2 <= 0.2))
	       group2.put(dub, temp2);
	    else if((d2 > 0.2) && (d2 <= 0.3))
	       group3.put(dub, temp2);
	    else if((d2 > 0.3) && (d2 <= 0.4))
	       group4.put(dub, temp2);
	    else if((d2 > 0.4) && (d2 <= 0.5))
	       group5.put(dub, temp2);
	    else if((d2 > 0.5) && (d2 <= 0.6))
	       group6.put(dub, temp2);
	    else if((d2 > 0.6) && (d2 <= 0.7))
	       group7.put(dub, temp2);
	    else if((d2 > 0.7) && (d2 <= 0.8))
	       group8.put(dub, temp2);
	    else if((d2 > 0.8) && (d2 <= 0.9))
	       group9.put(dub, temp2);
	    else
	       group10.put(dub, temp2);
	 }
		 
         //We now have 10 random Email Lists
         //And will write them to files
         FileOutputStream out1 = new FileOutputStream("List1");
	 ObjectOutputStream s1 = new ObjectOutputStream(out1);
	 s1.writeObject(group1);
	 s1.close();
	 out1.close();
		 
	 FileOutputStream out2 = new FileOutputStream("List2");
         ObjectOutputStream s2 = new ObjectOutputStream(out2);
         s2.writeObject(group2);
	 s2.close();
	 out2.close();
		 
	 FileOutputStream out3 = new FileOutputStream("List3");
	 ObjectOutputStream s3 = new ObjectOutputStream(out3);
	 s3.writeObject(group3);
         s3.close();
         out3.close();
		 
	 FileOutputStream out4 = new FileOutputStream("List4");
	 ObjectOutputStream s4 = new ObjectOutputStream(out4);
	 s4.writeObject(group4);
	 s4.close();
	 out4.close();
		 
         FileOutputStream out5 = new FileOutputStream("List5");
	 ObjectOutputStream s5 = new ObjectOutputStream(out5);
	 s5.writeObject(group5);
	 s5.close();
	 out5.close();
		 
	 FileOutputStream out6 = new FileOutputStream("List6");
	 ObjectOutputStream s6 = new ObjectOutputStream(out6);
         s6.writeObject(group6);
         s6.close();
	 out6.close();
		 
	 FileOutputStream out7 = new FileOutputStream("List7");
         ObjectOutputStream s7 = new ObjectOutputStream(out7);
	 s7.writeObject(group7);
	 s7.close();
	 out7.close();
		 
	 FileOutputStream out8 = new FileOutputStream("List8");
         ObjectOutputStream s8 = new ObjectOutputStream(out8);
	 s8.writeObject(group8);
	 s8.close();
	 out8.close();
		 
	 FileOutputStream out9 = new FileOutputStream("List9");
	 ObjectOutputStream s9 = new ObjectOutputStream(out9);
	 s9.writeObject(group9);
	 s9.close();
	 out9.close();
		 
	 FileOutputStream out10 = new FileOutputStream("List10");
	 ObjectOutputStream s10 = new ObjectOutputStream(out10);
	 s10.writeObject(group10);
	 s10.close();
	 out10.close();
      }
      catch(IOException e)
      {
         System.err.println(e);
      }
   }
}
