package spamfilter;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Handles groups of radio buttons.  Puts specified radio buttons
 * together on a panel, and associates them such that only one
 * at a time may be checked.
 * @author Tom Ashley
 * @author Matt Hansen
 */
public class RadioPanelGroup extends JPanel implements ActionListener {
     
	 protected JRadioButton move, mark, remove, POP, IMAP, Bayesian, Neural;
	 protected String serverType, filterMethod, spamAction;
	 protected JButton button;
	 
         /**
          * Instantiates a radio panel group.
          * @param server_type POP or IMAP
          * @param filter_method Bayesian or Neural.
          * @param spam_action mark, move or delete.
          * @param tmark radio button for the mark action
          * @param tmove radio button for the move action
          * @param tremove radio button for the delete action
          * @param tPOP radio button for POP protocol
          * @param tIMAP radio button for IMAP protocol
          * @param tBayesian radio button for Bayesian filtering
          * @param tNeural radio button for Neural network filtering,
          * which isn't implemented.
          * @param tbutton needed for associating the action listener
          */
	 public RadioPanelGroup(String server_type, String filter_method,
	                        String spam_action, 
				JRadioButton tmark, JRadioButton tmove, 
				JRadioButton tremove, JRadioButton tPOP,
				JRadioButton tIMAP, JRadioButton tBayesian,
				JRadioButton tNeural, JButton tbutton) {
		  super();
		  serverType = server_type;
		  filterMethod = filter_method;
		  spamAction = spam_action;
		  mark = tmark;
		  move = tmove;
		  remove = tremove;
		  POP = tPOP;
		  IMAP = tIMAP;
		  Bayesian = tBayesian;
		  Neural = tNeural;
		  button = tbutton;
		  createRadioPanelGroup(serverType, filterMethod, spamAction,
		                       mark, move, remove, POP, IMAP, Bayesian,
							   Neural);
	 }
	 
	 private void createRadioPanelGroup(String serverType, String filterMethod,
	                                    String spamAction, JRadioButton mark,
									 	JRadioButton move, JRadioButton remove,
										JRadioButton POP, JRadioButton IMAP, 
										JRadioButton Bayesian,
										JRadioButton Neural) {
										
		  
		  JPanel mainPanel = new JPanel(new GridBagLayout());
		  JPanel group1 = new JPanel(new GridLayout(0,1));
		  JPanel group2 = new JPanel(new GridLayout(0,1));
		  JPanel group3 = new JPanel(new GridLayout(0,1));
		  //group1.setLayout(new GridBagLayout());
		  GridBagConstraints c = new GridBagConstraints();
		  
		  ButtonGroup server = new ButtonGroup();
		  ButtonGroup filter = new ButtonGroup();
		  ButtonGroup action = new ButtonGroup();
		  
		  /*mark = new JRadioButton(spamActions[0]);
		  move = new JRadioButton(spamActions[1]);
		  remove = new JRadioButton(spamActions[2]);*/
		  if(spamAction.equals("Mark spam")) mark.setSelected(true);
		  else if(spamAction.equals("Move spam")) move.setSelected(true);
		  else remove.setSelected(true);
		  mark.setActionCommand("mark");
		  //move.setActionCommand("move");
		  remove.setActionCommand("delete");
		  mark.addActionListener(this);
		  move.addActionListener(this);
		  remove.addActionListener(this);
		  action.add(mark);
		  //action.add(move);
		  action.add(remove);
		  group3.add(mark);
		  //group3.add(move);
		  group3.add(remove);
		  
		  /*POP = new JRadioButton(serverLabels[0]);
		  IMAP = new JRadioButton(serverLabels[1]);*/
		  if(serverType.equals("POP")) {
		       POP.setSelected(true);
			   move.setEnabled(false);
		  }
		  else IMAP.setSelected(true);
		  POP.setActionCommand("POP");
		  IMAP.setActionCommand("IMAP");
		  POP.addActionListener(this);
		  IMAP.addActionListener(this);
		  server.add(POP);
		  server.add(IMAP);
		  group1.add(POP);
		  group1.add(IMAP);
		  
		  /*Bayesian = new JRadioButton(filterMethods[0]);
		  Neural = new JRadioButton(filterMethods[1]);*/
		  if(filterMethod.equals("Bayesian")) Bayesian.setSelected(true);
		  else
                  {
                     //Neural network filtering not implemented,
                     //so option disabled from the GUI
                     //Neural.setSelected(true);
                  }
		  Bayesian.setActionCommand("Bayesian");
		  Neural.setActionCommand("Neural");
                  //Disable the neural network button
                  Neural.setEnabled(false);
		  Bayesian.addActionListener(this);
		  Neural.addActionListener(this);
		  filter.add(Bayesian);
		  filter.add(Neural);
		  group2.add(Bayesian);
		  group2.add(Neural);
		  
		  group1.setPreferredSize(new Dimension(120, 100));
		  group1.setMinimumSize(new Dimension(120, 100));
		  group2.setPreferredSize(new Dimension(140, 100));
		  group2.setMinimumSize(new Dimension(140, 100));
		  group3.setPreferredSize(new Dimension(120, 100));
		  group3.setMinimumSize(new Dimension(120, 100));
		  
		  c.gridx = 0;
		  c.gridy = 0;
		  mainPanel.add(group1, c);
		  group1.setBorder(BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), 
			"Server Type", 
			TitledBorder.CENTER, 
			TitledBorder.TOP));
		  
		  c.gridx = 1;
		  c.gridy = 0;
		  mainPanel.add(group2, c);
		  group2.setBorder(BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), 
			"Filter Technique", 
			TitledBorder.CENTER, 
			TitledBorder.TOP));
		  
		  c.gridx = 2;
		  c.gridy = 0;
		  mainPanel.add(group3, c);
		  group3.setBorder(BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), 
			"Spam Action", 
			TitledBorder.CENTER, 
			TitledBorder.TOP));
		  
		  add(mainPanel);
	  }
	  
	public void actionPerformed(ActionEvent e) {
	  	button.setEnabled(true);
		if ("POP".equals(e.getActionCommand()))
        {
            if (move.isSelected())
                mark.setSelected(true);
            move.setEnabled(false);
        }
		
		 if ("IMAP".equals(e.getActionCommand()))
              move.setEnabled(true);
			  
		 if(POP.isSelected())
		 		serverType = "POP";
	     else serverType = "IMAP";
		 if(Bayesian.isSelected())
		 		filterMethod = "Bayesian";
		 else filterMethod = "Neural";
		 if(mark.isSelected())
		 		spamAction = "Mark spam";
		 else if(move.isSelected())
		 		spamAction = "Move spam";
		 else spamAction = "Delete spam";
    }
}
