/*
     SpamGUI.java by Matt Hansen, Tom Ashley, and Jen Knutson
*/
package spamfilter;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;
import java.lang.Object.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.net.*;
import java.nio.channels.*;

/**
 * Creates and displays the GUI for SpamBurglar. This class uses many other 
 * classes in SpamBurglar to create the config, help, and startup panels for 
 * SpamBurglar. It makes heavy use of the Swing API from java and also deals 
 * with all events that occur in any of the panels. The config panel modifies 
 * the Globals .spam.conf file to match the user's preferences and also 
 * contains the white list editing tool. The startup panel only conatins a 
 * button which enables the user to start, stop, or restart the proxy and 
 * initiate an email session. The help panel contains a lovely help file 
 * written by Tom Ashley.
 * @author Tom Ashley
 * @author Matt Hansen
 * @author Jen Knutson
 */
public class SpamGUI {
    public static Proxy proxy = null;
    
    public static final String confFile = ".spam.conf";

    private static String[] serverLabels = {"POP", "IMAP"};
    private static String[] filterMethods = {"Bayesian", "Neural Network"};
    private static String[] spamActions = {"Mark spam", "Move spam", "Delete spam"};
	 

    public static void main(String[] args) {
	//Read in the preferences file
	
	final Properties settings = new Properties();
        
        // Flag for detecting if .spam.conf is empty/non-existent
        boolean noInput = false;
	
	try {
            FileInputStream in = new FileInputStream(confFile);
	    settings.load(in);
	    in.close();
	} catch(IOException e) {
            noInput = true;
	    System.out.println("Configuration file doesn't exist; creating file: " +
		    confFile);
	}
		  
        // load properties, use defaults if no key found
	String serverType = settings.getProperty("ServerType","POP");
	String mailHost = settings.getProperty("MailServerHost","localhost");	
	String mPort = settings.getProperty("MailServerPort","110");
	String pPort = settings.getProperty("ProxyPort","5000");
        String filterMethod = settings.getProperty("FilterMethod","Bayesian");
        String spamAction = settings.getProperty("SpamAction","Mark spam");
	String threshold = settings.getProperty("Threshold","90");
        String train = settings.getProperty("Train","false");
		  
        final JTextField textField1 = new JTextField(mailHost, 15);
	final JTextField textField2 = new JTextField(mPort, 15);
        final JTextField textField3 = new JTextField(pPort, 15);
	final JRadioButton mark = new JRadioButton(spamActions[0]);

        // Move button is not currently functional, so not displayed
        // in the configuration window
        final JRadioButton move = new JRadioButton(spamActions[1]);
	final JRadioButton remove = new JRadioButton(spamActions[2]);
	final JRadioButton POP = new JRadioButton(serverLabels[0]);
	final JRadioButton IMAP = new JRadioButton(serverLabels[1]);
	final JRadioButton Bayesian = new JRadioButton(filterMethods[0]);
	final JRadioButton Neural = new JRadioButton(filterMethods[1]);
	final JCheckBox box1 = new JCheckBox();
		  
		  
	// Create top-level window
	JFrame mainWindow = new JFrame("SpamBurglar Preferences");
	mainWindow.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
	  
	// Create new tabbed pane
        JTabbedPane tabbedPane = new JTabbedPane();
		  
	// Tab #1 contents: Proxy settings
	JPanel configStuff = new JPanel();
	configStuff.setLayout(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	configStuff.setPreferredSize(new Dimension(500, 450));
	configStuff.setMinimumSize(new Dimension(267, 200));
	configStuff.setBorder(BorderFactory.createTitledBorder
		(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), 
						  "Configuration",
						  TitledBorder.LEFT,
						  TitledBorder.TOP));
		  
	boolean valuesChanged = false;
        final JButton button = new JButton("Save changes");
        final JButton WList = new JButton("Edit White List");

        MouseAdapter mouse = new MouseAdapter() {
           public void mouseClicked(MouseEvent e) {
              button.setEnabled(true);
           }
        };
		  
	textField1.addMouseListener(mouse);
	textField2.addMouseListener(mouse);
	textField3.addMouseListener(mouse);
		  	  
        JPanel textFieldGroup = new TextPanelGroup(mailHost, mPort, pPort,
		                           textField1, textField2, textField3);
	c.gridx = 0;
        c.gridy = 0;
	c.anchor = GridBagConstraints.LINE_START;
	configStuff.add(textFieldGroup, c);
		  
        JPanel radioPanelGroup = new RadioPanelGroup(serverType,
	   filterMethod, spamAction, mark, move, remove, POP, IMAP, Bayesian,
	   Neural, button);
        c.gridx = 0;
        c.gridy = 1;
	configStuff.add(radioPanelGroup, c);
		  
	Integer temp = new Integer(threshold);
        int thresh = temp.intValue();
        SpinnerModel model = new SpinnerNumberModel(thresh, 0, 100, 1);
	final JSpinner spinner1 = new JSpinner(model);
        JPanel threshhold = new SpinnerPanel("Spam Threshhold %", spinner1);
        spinner1.addChangeListener(new ChangeListener() {
	   public void stateChanged(ChangeEvent e)
	   {
	      button.setEnabled(true);
           }
	});

	c.gridx = 0;
	c.gridy = 2;
	c.anchor = GridBagConstraints.LINE_START;
	configStuff.add(threshhold, c);
		  
	boolean tMode;
	if (train.equals("true")) tMode = true;
	else tMode = false;
		  
	JPanel trainingMode = new CheckBoxPanel("Training Mode", box1,
		                                 tMode, button);
        c.gridx = 0;
        c.gridy = 2;
	c.anchor = GridBagConstraints.LINE_END;
        configStuff.add(trainingMode, c);
		  
        button.setActionCommand("Save changes");
        button.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e)
	   {
	      String t = new String();
	      t = t.valueOf(spinner1.getValue());
	      settings.setProperty("MailServerHost", textField1.getText());
	      settings.setProperty("MailServerPort", textField2.getText());
	      settings.setProperty("ProxyPort", textField3.getText());
	      if(POP.isSelected())
	         settings.setProperty("ServerType", "POP");
	      else
		 settings.setProperty("ServerType", "IMAP");
	      if(Bayesian.isSelected())
		 settings.setProperty("FilterMethod", "Bayesian");
	      else
		 settings.setProperty("FilterMethod", "Neural network");
	      if(mark.isSelected())
		 settings.setProperty("SpamAction", "Mark spam");
	      else if(move.isSelected())
		 settings.setProperty("SpamAction", "Move spam");
	      else 
		settings.setProperty("SpamAction", "Delete spam");
	      settings.setProperty("Threshold", t);
	      if(box1.isSelected())
	         settings.setProperty("Train", "true"); 
	      else
		 settings.setProperty("Train", "false");
				   
	      //Disable the "save changes" button until a change is made
	      button.setEnabled(false);
	      //Write changes to the config file
	      try {
	         FileOutputStream out = new FileOutputStream(confFile);
		 settings.store(out, "User preferences for SpamBurglar");
		 out.close();
	      } catch(IOException exc) {
                 System.out.println("Unable to write changes to file:" + exc);
	      }
	   }
	});
        
        c.gridx = 0;
        c.gridy = 3;
	c.anchor = GridBagConstraints.LINE_END;
        configStuff.add(button, c);

        WList.setActionCommand("Edit White List");
        WList.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e)
           {
              WLGUI.createAndShowGUI();
           }
        });

        c.gridx = 0;
        c.gridy = 3;
        c.anchor = GridBagConstraints.LINE_START;
        configStuff.add(WList, c);
		  

        // Tab #2 contents: Proxy startup window
	final JPanel proxyStartup = new JPanel();
	proxyStartup.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		  		 
	// 2/2/04: jen
        // ------------------------------------------------------
        final JButton button2 = new JButton("Start Proxy Server");
	button2.setActionCommand("START");
        button2.setMargin(new Insets(50,50,50,50));
	ActionListener startstop = new ActionListener(){
	   public void actionPerformed(ActionEvent e) {
	      try {
	         if(e.getActionCommand().equals("START")) {
		    Globals.load(confFile);				  
		    proxy = new Proxy(Globals.proxyPort,Globals.localhostOnly);
				  
		    if(proxy != null) {
		       proxy.start();
		       button2.setText("Stop Proxy Server");
		       button2.setActionCommand("STOP");
		    }
		 }
	         else { // "STOP"
		    if(proxy.numConnections() == 0) {
		       proxy.interrupt();
                       proxy = null;
		       button2.setText("Restart Proxy Server");
		       button2.setActionCommand("START");
		    }
                    else {
	               int n = JOptionPane.showConfirmDialog(new JFrame(),
		          "Are you sure you want to close the proxy" +
			  " server?\nIt is recommended that you logout"+ 
			  " of all client connections before shutting" +
                          " down.", "Connections still open",
	                  JOptionPane.YES_NO_OPTION);
		       if (n == JOptionPane.YES_OPTION) {
		          proxy.interrupt();
                          proxy = null;
			  button2.setText("Restart Proxy Server");
			  button2.setActionCommand("START");
		       }	   
	            }
		 }
	      } catch (NotYetBoundException x) { // proxy didn't initialize
	      } catch (UnresolvedAddressException x) {
	       System.err.println("Could not resolve mail server's address.");
	      } catch (Exception x) {	
	         System.err.println(x.getMessage());
	      }
	   }		  
	};
	
        WindowAdapter window = new WindowAdapter() {
           public void windowClosing(WindowEvent e) {
              try{
                 if (proxy != null)
                 {
                    JOptionPane.showMessageDialog(new JFrame(),
                       "Must shut down the proxy server before exiting",
                       "Proxy server still running",
                       JOptionPane.WARNING_MESSAGE);
                 }
                 else
                    System.exit(0);
              }
              catch (NotYetBoundException x) { // proxy didn't initialize
	      } catch (UnresolvedAddressException x) {
	       System.err.println("Could not resolve mail server's address.");
	      } catch (Exception x) {	
	       System.err.println(x.getMessage());
	      }
           }
        };
	mainWindow.addWindowListener(window);
	  
        button2.addActionListener(startstop);
        proxyStartup.add(button2, c);
        // ------------------------------------------------------    
		  
	// Tab #3 Contents: Help system
        String helpFile = "";
        try {
           File inputFile = new File("spamfilter/aboutFile");
           FileReader in2 = new FileReader(inputFile);
           int check;
           while ((check = in2.read()) != -1)
           {
              char temp2 = (char) check;
              helpFile += temp2;
           }
           in2.close();
        } catch (IOException e) {
           System.out.println(e);
           helpFile = "Unable to read the SpamBurglar help file";
        }

        final JPanel aboutSB = new JPanel();
	aboutSB.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
        JTextArea textArea = new JTextArea(helpFile,20,40);
        JScrollPane scrollPane = new JScrollPane(textArea);
	c.anchor = GridBagConstraints.CENTER;        
        aboutSB.add(scrollPane, c);
        
	// Add tab contents to tabbedPane
        if (noInput)
        {
           tabbedPane.addTab("Configuration", null, configStuff, "Settings");
	   tabbedPane.addTab("Proxy startup", null, proxyStartup,
                             "Start server");
           tabbedPane.addTab("About SpamBurglar", null, aboutSB, "About...");
        }
        else
        {
           tabbedPane.addTab("Proxy startup", null, proxyStartup,
                             "Start server");
           tabbedPane.addTab("Configuration", null, configStuff, "Settings");
           tabbedPane.addTab("About SpamBurglar", null, aboutSB, "About...");
        }
			
        // Add tabbed pane to root window
        mainWindow.getContentPane().add(tabbedPane, BorderLayout.CENTER);
        // Display the window
	mainWindow.pack();
        mainWindow.setVisible(true);
     }
}
