// TO DO:
// * Warn with a dialog on a 'Cancel'
// * Make prettier
// * Allow individual emails to be ignored (i.e. not used in training)
// * Tooltips?
// * Should we show part of message body, too, in a little pane???
// --jen

package spamfilter;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;

public class TrainingWindow extends JPanel implements ActionListener {

    private static final int UPDATE_BUTTON = 0;
    private static final int CANCEL_BUTTON = 1;
    private final Vector sessionMail;
    private final JFrame tw;

    // Format for displaying Email objects in a table
    private class EmailTable extends JPanel {
	private final Vector emails;

	// constructor
	public EmailTable(Vector mail) {
	    super(); 
	    emails = mail;

	    final EmailTableModel model = new EmailTableModel();
	    final JTable table = new JTable(model);
	    setFormat(table);
	    JScrollPane sp = new JScrollPane(table);	    

	    // add listener for clicks on column headers (e.g. to sort)
	    MouseAdapter mouse = new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    int clickedColumn =
			table.convertColumnIndexToModel(
				table.getColumnModel().getColumnIndexAtX(
				    e.getX()));
		    if(e.getClickCount() == 1 && clickedColumn != -1) {
			boolean order = ((e.getModifiers() &
				    InputEvent.SHIFT_MASK) == 0);
			model.sort(clickedColumn, order);
		    }
		}
	    };
	    table.getTableHeader().addMouseListener(mouse);

	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    setBorder(BorderFactory.createEmptyBorder(10,10,10,10));  
	    add(sp);
	}

	// Set preferred sizes, display options for EmailTable
	private void setFormat(final JTable table) {
	    table.setPreferredScrollableViewportSize(new Dimension(700,200));
	    table.setShowVerticalLines(false);

	    table.getColumnModel().getColumn(0).setPreferredWidth(50);
	    table.getColumnModel().getColumn(0).setMaxWidth(50);
	    table.getColumnModel().getColumn(0).setMinWidth(50);
	    table.getColumnModel().getColumn(1).setPreferredWidth(200);
	    table.getColumnModel().getColumn(2).setPreferredWidth(400);
	}

	// Set up model for displaying table
	private class EmailTableModel extends AbstractTableModel {
	    private String[] columnNames = {"Spam", "Sender", "Subject"};

	    public int getColumnCount() { return columnNames.length; }
	    public int getRowCount() { return emails.size(); }
	    public String getColumnName(int col) { return columnNames[col]; }

	    public Object getValueAt(int row, int col) {
		Email e = (Email) emails.get(row);
		switch(col) {
		    case 0: return new Boolean(e.isSpam());
		    case 1: return e.getSender();
		    case 2: return e.getSubject();
		    default: return null;
		}
	    }

	    public Class getColumnClass(int c) {
		return getValueAt(0, c).getClass();
	    }

	    public boolean isCellEditable(int row, int col) {
		if(col > 0)	return false;
		else 		return true;
	    }

	    public void setValueAt(Object value, int row, int col) {
		if(isCellEditable(row, col)) {
		    Email e = (Email) emails.get(row);
		    if(((Boolean)value).booleanValue())
			e.markAsSpam();
		    else
			e.markAsNonspam();		    
		}
	    }


	    // A way to sort Email objects appropriately by columns in table
	    private void sort(final int column, final boolean ascending) {
		Comparator sorter = new Comparator() {	
		    public int compare(Object o1, Object o2) {
			int value = 0;
			if(column == 0) {
			    boolean a = ((Email)o1).isSpam();
			    boolean b = ((Email)o2).isSpam();
			    if(!a && b)
				value = 1;
			    else if(a && !b)
				value = -1;
			    else
				value = ((Email)o1).getSender().toLowerCase().
				    compareTo(
					    ((Email)o2).getSender().toLowerCase());
			}
			else if(column == 1)
			    value = ((Email)o1).getSender().toLowerCase().
				compareTo(
					((Email)o2).getSender().toLowerCase());
			else if(column == 2)
			    value = ((Email)o1).getSubject().toLowerCase().
				compareTo(
					((Email)o2).getSubject().toLowerCase());

			if(ascending)
			    return value;
			else
			    return -value;
		    }		    
		};

		Collections.sort(emails, sorter);
	    }
	}
    }


    // listens for button events
    public void actionPerformed(ActionEvent e) {

	tw.setVisible(false);
	if("UPDATE".equals(e.getActionCommand())) {
	    System.out.println("NOW CALLING CORPUS UPDATE METHOD" + 
		    " USING emails");
	    SpamIdentifier.updateCorpus(sessionMail);
	} else if("CANCEL".equals(e.getActionCommand())) {
	    System.out.println("CORPUS WILL NOT BE UPDATED");	
	    System.out.println("HAVE A CONFIRMATION DIALOG HERE???");
	}

	// Close the main window
	tw.dispose();
    }

    // easy way to set up a particular button type
    private JButton makeButton(int type) {
	JButton button = null;
	if(type == UPDATE_BUTTON) {
	    button = new JButton("Train SpamBurglar");
	    button.setActionCommand("UPDATE");
	} else if(type == CANCEL_BUTTON) {
	    button = new JButton("Cancel");
	    button.setActionCommand("CANCEL");
	} else
	    button = new JButton();

	button.addActionListener(this);
	return button;
    }


    // The full display of information, including email list & buttons
    public TrainingWindow(Vector emails) {

	sessionMail = emails;
	tw = new JFrame("SpamBurglar Training Session");	

	tw.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
	buttonPanel.add(Box.createHorizontalGlue());
	buttonPanel.add(makeButton(CANCEL_BUTTON));
	buttonPanel.add(Box.createRigidArea(new Dimension(10,0)));
	buttonPanel.add(makeButton(UPDATE_BUTTON));	

	tw.getContentPane().add(new EmailTable(emails), BorderLayout.CENTER);
	tw.getContentPane().add(buttonPanel, BorderLayout.PAGE_END);
	tw.pack();
	tw.setVisible(true);
    }
}
