package spamfilter;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;

/* Modified ListDemo.java from java.sun.com by Matt Hansen*/
/**
 * A GUI for manipulating the white list. Most of this code is borrowed from 
 * ListDemo.java from java.sun.com.
 * @author Matt Hansen
 */

public class WLGUI extends JPanel
                      implements ListSelectionListener {
    private JList list;
    private DefaultListModel listModel;

    private static final String addString = "Add";
    private static final String removeString = "Remove";
    private JButton removeButton;
    private JTextField newAddress;
	private Hashtable whiteList = null;

    /**
	 * Constructor class for WLGUI. This method reads in the white list from
	 * Globals.whiteListFile using a hashtable and enumerations, or creates 
	 * a blank white list, then sets up the GUI which the user can then 
	 * manipulate.
	 */
	public WLGUI() {
        super(new BorderLayout());

        //try to read whitelist
		try {
 	        FileInputStream in = new FileInputStream(Globals.whiteListFile);
	        ObjectInputStream s = new ObjectInputStream(in);
			whiteList = (Hashtable) s.readObject();
	 		s.close();
	 		in.close();
 		}
		catch(FileNotFoundException e) {
		 try{ 
		  System.out.println("White List missing: Creating new White List now");
		  ObjectOutputStream out = new ObjectOutputStream(
		  	  new FileOutputStream(Globals.whiteListFile));
		  Hashtable temp = new Hashtable();
		  out.writeObject(temp);
		  out.close();
		 } catch(IOException ex) {
		 	System.err.println(ex);
		  }
		}
        catch(IOException e) {
			System.err.println(e);
        }
        catch(ClassNotFoundException e) {
	   		System.err.println(e);
        }
		
		Enumeration items = whiteList.keys();
		
		listModel = new DefaultListModel();
		while(items.hasMoreElements())
		{
			listModel.addElement((String) items.nextElement());
		}
		

        //Create the list and put it in a scroll pane.
        list = new JList(listModel);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        list.setSelectedIndex(0);
        list.addListSelectionListener(this);
        list.setVisibleRowCount(5);
        JScrollPane listScrollPane = new JScrollPane(list);

        JButton addButton = new JButton(addString);
        AddListener addListener = new AddListener(addButton);
        addButton.setActionCommand(addString);
        addButton.addActionListener(addListener);
        addButton.setEnabled(false);

        removeButton = new JButton(removeString);
        removeButton.setActionCommand(removeString);
        removeButton.addActionListener(new RemoveListener());

        newAddress = new JTextField(10);
        newAddress.addActionListener(addListener);
        newAddress.getDocument().addDocumentListener(addListener);
        String name = "";
		int size = listModel.size();
		if(size != 0)
		{
			name = listModel.getElementAt(
                              list.getSelectedIndex()).toString();
		}
		else
			removeButton.setEnabled(false);
		
        //Create a panel that uses BoxLayout.
        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new BoxLayout(buttonPane,
                                           BoxLayout.LINE_AXIS));
        buttonPane.add(removeButton);
        buttonPane.add(Box.createHorizontalStrut(5));
        buttonPane.add(new JSeparator(SwingConstants.VERTICAL));
        buttonPane.add(Box.createHorizontalStrut(5));
        buttonPane.add(newAddress);
        buttonPane.add(addButton);
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

        add(listScrollPane, BorderLayout.CENTER);
        add(buttonPane, BorderLayout.PAGE_END);
    }

    /**
	* An action listener for the Remove button. This class can only be called 
	* if there is a valid selection. It removes whatever is selected from both 
	* the GUI window and the white list.
	* @author Matt Hansen
	*/
	class RemoveListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            //This method can be called only if
            //there's a valid selection
            //so go ahead and remove whatever's selected.
            int index = list.getSelectedIndex();
			String rAddress = (String) list.getSelectedValue();
			whiteList.remove(rAddress);
			try{
				FileOutputStream out = 
				           new FileOutputStream(Globals.whiteListFile);
     			ObjectOutputStream oOut = new ObjectOutputStream(out);
     			oOut.writeObject(whiteList);
     			oOut.close();
     			out.close();
			} catch(IOException er) {
				System.err.println(er);
				System.exit(-1);
			}
            listModel.remove(index);

            int size = listModel.getSize();

            if (size == 0) { //Nobody's left, disable firing.
                removeButton.setEnabled(false);

            } else { //Select an index.
                if (index == listModel.getSize()) {
                    //removed item in last position
                    index--;
                }

                list.setSelectedIndex(index);
                list.ensureIndexIsVisible(index);
            }
        }
    }

    /**
	* An Action Listener for the add button and text field. This class is 
	* called when the Add button is clicked. It reads whatever's in the text 
	* field, places it on the white list and in the text box, and removes it 
	* from the text field. It checks whether the text is already in the text 
	* box or whether no text was entered, but doe not check validity beyond 
	* that. 
	* @author Matt Hansen
	*/
	//This listener is shared by the text field and the hire button.
    class AddListener implements ActionListener, DocumentListener {
        private boolean alreadyEnabled = false;
        private JButton button;

        public AddListener(JButton button) {
            this.button = button;
        }

        //Required by ActionListener.
        public void actionPerformed(ActionEvent e) {
            String name = newAddress.getText();

            //User didn't type in a unique name...
            if (name.equals("") || alreadyInList(name)) {
                Toolkit.getDefaultToolkit().beep();
                newAddress.requestFocusInWindow();
                newAddress.selectAll();
                return;
            }

            listModel.addElement(newAddress.getText());
			
			whiteList.put(newAddress.getText(), "white");
			try{
				FileOutputStream out2 = 
				           new FileOutputStream(Globals.whiteListFile);
     			ObjectOutputStream oOut2 = new ObjectOutputStream(out2);
     			oOut2.writeObject(whiteList);
     			oOut2.close();
     			out2.close();
			} catch(IOException ex) {
				System.err.println(ex);
				System.exit(-1);
			}
			int size = listModel.getSize();
			list.setSelectedIndex(size-1);

            //Reset the text field.
            newAddress.requestFocusInWindow();
            newAddress.setText("");
        }

        //This method tests for string equality. You could certainly
        //get more sophisticated about the algorithm.  For example,
        //you might want to ignore white space and capitalization.
        protected boolean alreadyInList(String name) {
            return listModel.contains(name);
        }

        //Required by DocumentListener.
        public void insertUpdate(DocumentEvent e) {
            enableButton();
        }

        //Required by DocumentListener.
        public void removeUpdate(DocumentEvent e) {
            handleEmptyTextField(e);
        }

        //Required by DocumentListener.
        public void changedUpdate(DocumentEvent e) {
            if (!handleEmptyTextField(e)) {
                enableButton();
            }
        }

        private void enableButton() {
            if (!alreadyEnabled) {
                button.setEnabled(true);
            }
        }

        private boolean handleEmptyTextField(DocumentEvent e) {
            if (e.getDocument().getLength() <= 0) {
                button.setEnabled(false);
                alreadyEnabled = false;
                return true;
            }
            return false;
        }
    }

    //This method is required by ListSelectionListener.
    public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting() == false) {

            if (list.getSelectedIndex() == -1) {
            //No selection, disable fire button.
                removeButton.setEnabled(false);

            } else {
            //Selection, enable the fire button.
                removeButton.setEnabled(true);
            }
        }
    }

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    public static void createAndShowGUI() {
        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(false);

        //Create and set up the window.
        JFrame frame = new JFrame("White List");
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

        //Create and set up the content pane.
        JComponent newContentPane = new WLGUI();
        newContentPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(newContentPane);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }

    /**
	* This method invokes the Swing library and runs CreateAndShowGUI. 
	* @param args this method does not take arguments from the command line.
	*/
	public static void main(String[] args) {
        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }
}
